@@ -11,11 +11,6 @@ def default_cache_func(
         django_model,
         cache_conf,
 ):
-    django_to_cache(
-        redis_root,
-        django_model,
-        cache_conf,
-    )
     cache_to_django(
         redis_root,
         django_model,
@@ -295,82 +290,3 @@ def django_sync_to_async_get(
     return param
 
 
-def django_to_cache(
-        redis_root,
-        django_model,
-        cache_conf,
-):
-    async def run_django_to_cache(
-            redis_root,
-            django_model,
-            django_fields,
-            cache_conf,
-    ):
-        filter_by = cache_conf['filter_by']
-        if filter_by:
-            django_instances = await django_sync_to_async_list(django_model.objects.filter, filter_by)
-        else:
-            django_instances = await django_sync_to_async_list(django_model.objects.all)
-        async_chunk_size = redis_root.async_db_requests_limit
-        async_tasks = []
-        completed = 0
-        to_complete = len(django_instances)
-        if django_instances:
-
-            for django_instance in django_instances:
-                async_tasks.append(
-                    update_or_create_redis_instance_from_django_instance(
-                        redis_root,
-                        django_instance,
-                        django_fields,
-                        cache_conf,
-                    )
-                )
-                completed += 1
-                if len(async_tasks) == async_chunk_size or completed == to_complete:
-                    await asyncio.gather(*async_tasks)
-                    async_tasks = []
-                    print(
-                        f'{datetime.datetime.now()} - '
-                        f'Written {completed} {django_model} instances from django to cache')
-
-        if cache_conf['delete']:
-            async def check_if_need_to_delete_redis_instance(redis_root, django_instances_ids, redis_instance):
-                if 'django_id' in redis_instance.keys():
-                    if redis_instance['django_id'] not in django_instances_ids:
-                        redis_root.delete(django_model, redis_instance)
-
-            django_instances_ids = [django_instance.id for django_instance in django_instances]
-            all_redis_instances = redis_root.get(django_model)
-            async_tasks = [
-                check_if_need_to_delete_redis_instance(redis_root, django_instances_ids, redis_instance)
-                for redis_instance in all_redis_instances
-            ]
-
-            await asyncio.gather(*async_tasks)
-
-        print(f'{datetime.datetime.now()} - '
-              f'Deleted {django_model} instances from cache')
-
-    django_fields = django_model._meta.get_fields()
-
-    loop = asyncio.get_event_loop()
-    loop.run_until_complete(
-        run_django_to_cache(
-            redis_root,
-            django_model,
-            django_fields,
-            cache_conf,
-        )
-    )
-
-
-
-
-
-
-
-
-
-
-
