@@ -1,22 +1,16 @@
 import datetime
 import decimal
 import json
-
+import asyncio
 import pytz
 import redis
 from copy import deepcopy
 from typing import Callable
 
-from django.conf import settings
-from django.core import exceptions
-from django.db import models as django_models, IntegrityError
-from django.db.models.query import ModelIterable
-from django.db.models.signals import post_save, pre_delete
-from django.db.models.utils import resolve_callables
-from django.dispatch import receiver
+from django.db import models as django_models
 
 from .utils import check_types, check_classes, get_ids_from_untyped_data
-from .cache import default_cache_func
+from .cache import default_cache_func, django_sync_to_async_list
 
 
 class RedisField:
@@ -465,18 +459,88 @@ class RedisRoot:
             cache_conf = self.registered_django_models[django_model]['cache_conf']
             return cache_conf
         else:
-            cache_conf = {
-                'enabled': False,
-                'ttl': 60 * 5,
-                'save_related_models': True,
-                'cache_func': default_cache_func,
-                'exclude_fields': [],
-                'filter_by': {},
-                'write_to_django': False,
-                'delete': False,
-            }
+            cached_cache_conf = self.redis_instance.get(f'{self.prefix}:cache_confs:{django_model.__name__}')
+            if cached_cache_conf:
+                cache_conf = cached_cache_conf
+            else:
+                cache_conf = {
+                    'enabled': False,
+                    'ttl': 60 * 5,
+                    'save_related_models': True,
+                    'exclude_fields': [],
+                    'filter_by': {},
+                    'write_to_django': False,
+                    'delete': False,
+                }
             return cache_conf
 
+    def django_to_cache(
+            self,
+    ):
+        async def run_django_to_cache(
+                redis_root,
+                django_model,
+                django_fields,
+                cache_conf,
+        ):
+            filter_by = cache_conf['filter_by']
+            if filter_by:
+                django_instances = await django_sync_to_async_list(django_model.objects.filter, filter_by)
+            else:
+                django_instances = await django_sync_to_async_list(django_model.objects.all)
+            async_chunk_size = redis_root.async_db_requests_limit
+            async_tasks = []
+            completed = 0
+            to_complete = len(django_instances)
+            if django_instances:
+
+                for django_instance in django_instances:
+                    async_tasks.append(
+                        self.update_or_create_redis_instance_from_django_instance(
+                            django_instance,
+                            django_fields,
+                            cache_conf,
+                        )
+                    )
+                    completed += 1
+                    if len(async_tasks) == async_chunk_size or completed == to_complete:
+                        await asyncio.gather(*async_tasks)
+                        async_tasks = []
+                        print(
+                            f'{datetime.datetime.now()} - '
+                            f'Written {completed} {django_model} instances from django to cache')
+
+            if cache_conf['delete']:
+                async def check_if_need_to_delete_redis_instance(redis_root, django_instances_ids, redis_instance):
+                    if 'django_id' in redis_instance.keys():
+                        if redis_instance['django_id'] not in django_instances_ids:
+                            redis_root.delete(django_model, redis_instance)
+
+                django_instances_ids = [django_instance.id for django_instance in django_instances]
+                all_redis_instances = redis_root.get(django_model)
+                async_tasks = [
+                    check_if_need_to_delete_redis_instance(redis_root, django_instances_ids, redis_instance)
+                    for redis_instance in all_redis_instances
+                ]
+
+                await asyncio.gather(*async_tasks)
+
+            print(f'{datetime.datetime.now()} - '
+                  f'Deleted {django_model} instances from cache')
+
+        for django_model, django_model_cache_data in self.registered_django_models.items():
+            cache_conf = django_model_cache_data['cache_conf']
+            django_fields = django_model._meta.get_fields()
+            loop = asyncio.get_event_loop()
+            loop.run_until_complete(
+                run_django_to_cache(
+                    self,
+                    django_model,
+                    django_fields,
+                    cache_conf,
+                )
+            )
+
     def django_fields_to_redis_fields(self, django_model, save_related_models, exclude_fields):
         redis_fields = {}
         for django_field in django_model._meta.get_fields():
@@ -609,9 +673,8 @@ class RedisRoot:
                   f'Start caching {django_model}'
                   f'\n')
 
-            cache_func = cache_conf['cache_func']
             redis_model = self.get_or_create_redis_model_from_django_model(django_model)
-            cache_func(
+            default_cache_func(
                 self,
                 django_model,
                 cache_conf,
@@ -664,7 +727,6 @@ class RedisRoot:
             'enabled': False,
             'ttl': 60 * 5,
             'save_related_models': True,
-            'cache_func': default_cache_func,
             'exclude_fields': [],
             'filter_by': {},
             'write_to_django': False,
@@ -687,12 +749,6 @@ class RedisRoot:
                         cache_conf['save_related_models'] = user_cache_conf['save_related_models']
                     else:
                         raise Exception(f'{name} -> cache config -> save_related_models must be int')
-                if 'cache_func' in user_cache_conf.keys():
-                    if type(user_cache_conf['cache_func']) == Callable:
-                        cache_conf['cache_func'] = user_cache_conf['cache_func']
-                    else:
-                        raise Exception(
-                            f'{name} -> cache config -> cache_func must be callable function')
                 if 'exclude_fields' in user_cache_conf.keys():
                     if type(user_cache_conf['exclude_fields']) == list:
                         if all([
@@ -730,7 +786,9 @@ class RedisRoot:
 
         return cache_conf
 
-    def update_or_create_redis_instance_from_django_instance(
+
+
+    async def update_or_create_redis_instance_from_django_instance(
             self,
             django_instance,
             django_fields,
@@ -751,7 +809,7 @@ class RedisRoot:
 
         django_instance_model = django_instance.__class__
         old_redis_instance = self.get(django_instance_model, django_id=django_instance.id)
-        redis_dict = self.django_instance_to_redis_params(
+        redis_dict = await self.django_instance_to_redis_params(
             django_instance,
             django_fields,
             cache_conf,
@@ -782,7 +840,7 @@ class RedisRoot:
 
         return result_redis_instance_id
 
-    def django_instance_to_redis_params(
+    async def django_instance_to_redis_params(
             self,
             django_instance,
             django_fields,
@@ -800,7 +858,7 @@ class RedisRoot:
                     allowed = (django_field.name not in exclude_fields)
                 if allowed:
                     if django_field.name not in ['id', 'pk']:
-                        redis_param = self.django_instance_field_to_redis_value(
+                        redis_param = await self.django_instance_field_to_redis_value(
                             django_instance,
                             django_field,
                             cache_conf,
@@ -810,7 +868,7 @@ class RedisRoot:
 
         return redis_params
 
-    def django_instance_field_to_redis_value(
+    async def django_instance_field_to_redis_value(
             self,
             django_instance,
             django_field,
@@ -818,7 +876,7 @@ class RedisRoot:
     ):
 
         save_related_models = cache_conf['save_related_models']
-        django_field_value = getattr(django_instance, django_field.name, None)
+        django_field_value = await django_sync_to_async_get(django_instance, django_field.name)
         redis_value = django_field_value
         if django_field.__class__ == django_models.ForeignKey:
             if django_field_value is None:
@@ -828,7 +886,7 @@ class RedisRoot:
                 if save_related_models:
                     if django_foreign_key_instance:
                         django_instance_model = django_foreign_key_instance.__class__
-                        redis_foreign_key_instance = self.get_or_create_redis_instance_from_django_instance(
+                        redis_foreign_key_instance = await self.get_or_create_redis_instance_from_django_instance(
                             django_foreign_key_instance,
                             cache_conf,
                         )
@@ -843,7 +901,7 @@ class RedisRoot:
                 if len(django_many_to_many_instances):
                     django_instance_model = django_many_to_many_instances[0].__class__
                     redis_value = [
-                        self.get_or_create_redis_instance_from_django_instance(
+                        await self.get_or_create_redis_instance_from_django_instance(
                             django_many_to_many_instance,
                             cache_conf,
                         )
@@ -863,7 +921,7 @@ class RedisRoot:
                 redis_value = None
         return redis_value
 
-    def get_or_create_redis_instance_from_django_instance(
+    async def get_or_create_redis_instance_from_django_instance(
             self,
             django_instance,
             cache_conf,
@@ -873,7 +931,7 @@ class RedisRoot:
         redis_instance = self.get(django_instance_model, django_id=django_instance.id)
 
         if not redis_instance:
-            redis_dict = self.django_instance_to_redis_params(
+            redis_dict = await self.django_instance_to_redis_params(
                 django_instance,
                 django_fields,
                 cache_conf,
@@ -893,28 +951,11 @@ class RedisRoot:
             if issubclass(model, django_models.Model):
                 if model not in self.registered_django_models.keys():
                     cache_conf = self.check_cache_conf(model, user_cache_conf)
+                    self.redis_instance.set(f'{self.prefix}:cache_confs:{model.__name__}', json.dumps(cache_conf))
                     self.registered_django_models[model] = {
                         'cache_conf': cache_conf,
                         'to_cache_in': datetime.datetime.now()
                     }
-
-                    @receiver(post_save, sender=model)
-                    def save_redis_instance_from_django_instance(sender, instance, **kwargs):
-                        django_fields = sender._meta.get_fields()
-                        self.update_or_create_redis_instance_from_django_instance(
-                            instance,
-                            django_fields,
-                            cache_conf
-                        )
-                        print('Called save_redis_instance_from_django_instance')
-
-                    @receiver(pre_delete, sender=model)
-                    def delete_redis_instance_from_django_instance(sender, instance, **kwargs):
-                        redis_instances_qs = self.get(sender, django_id=instance.id)
-                        if redis_instances_qs:
-                            self.delete(sender, redis_instances_qs)
-                        print('Called delete_redis_instance_from_django_instance')
-
             else:
                 raise Exception(f'{model.__name__} class is not django_models.Model')
 
@@ -1206,7 +1247,7 @@ class RedisRoot:
             }
         return updated_data
 
-    def update(self, django_model, instances=None, return_dict=False, renew_ttl=False, new_ttl=None,
+    def update(self, django_model, instanes=None, return_dict=False, renew_ttl=False, new_ttl=None,
                **fields_to_update):
         model = self.get_or_create_redis_model_from_django_model(django_model)
         model_name = model.__name__
@@ -1422,44 +1463,3 @@ class RedisModel:
             raise Exception(f'{name} has no field {field_name}')
 
 
-class RedisQuerySet:
-
-    def __init__(self, model=None, query=None):
-        self.model = model
-
-    def get(self, **filters):
-        return self.model.redis_root.get(self.model, **filters)
-
-    def update(self, instances, **params):
-        return self.model.redis_root.update(self.model, instances, **params)
-
-    def create(self, **params):
-        return self.model.save(**params)
-
-    def delete(self, instances):
-        return self.model.redis_root.delete(self.model, instances)
-
-
-class RedisObjectsManager(django_models.Manager):
-    def get_queryset(self, **kwargs):
-        return RedisQuerySet(self)
-
-
-class DjangoRedisModel(django_models.Model):
-    objects = RedisObjectsManager()
-    django_objects = django_models.Manager()
-    redis_root = settings.REDIS_ROOT
-
-    def save(self, *args, **kwargs):
-        existing_fields = {
-            field.name: getattr(self, field.name)
-            for field in self.__class__._meta.get_fields()  # Parents included
-            if field.name not in ['objects', 'django_objects', 'redis_root']
-        }
-        return self.redis_root.create(self.__class__, **existing_fields)
-
-    def django_save(self, *args, **kwargs):
-        return super().save(*args, **kwargs)
-
-    class Meta:
-        abstract = True
